#! /usr/local/bin/ruby

prms = <<EOS
CURLOPT_URL              string
CURLOPT_PROXY            string
CURLOPT_PROXYPORT        long
CURLOPT_USERPWD          string
CURLOPT_USERAGENT        string
CURLOPT_COOKIE           string
CURLOPT_POSTFIELDS       string
CURLOPT_FOLLOWLOCATION   bool
CURLOPT_TIMEOUT          long

CURLOPT_PROXYUSERPWD	string	'user:passwd'
CURLOPT_HTTPPROXYTUNNEL	bool
CURLOPT_FAILONERROR	bool
CURLOPT_REFERER		string
CURLOPT_SSLCERT		string	'PEMfilename'
CURLOPT_SSLCERTPASSWD	string	'password'
CURLOPT_SSLVERSION	long
CURLOPT_COOKIEFILE	string	'cookieFile'
CURLOPT_CUSTOMREQUEST	string
CURLOPT_TIMECONDITION	long	enum
CURLOPT_TIMEVALUE	long	in seconds
CURLOPT_MAXREDIRS	long	redirection limit

CURLOPT_NETRC		bool
CURLOPT_FTPLISTONLY	bool
CURLOPT_FTPAPPEND	bool
CURLOPT_TRANSFERTEXT	bool
CURLOPT_RANGE		string	'100-200,250-300'
CURLOPT_FTPPORT		string
CURLOPT_LOW_SPEED_LIMIT	long	in bytes per second
CURLOPT_LOW_SPEED_TIME	long	in seconds
CURLOPT_RESUME_FROM	long	offset in number of bytes
CURLOPT_CRLF		bool

CURLOPT_INTERFACE	string
CURLOPT_KRB4LEVEL	string
CURLOPT_SSL_VERIFYPEER	bool
CURLOPT_CAINFO		string	'filename'

CURLOPT_VERBOSE		bool
CURLOPT_HTTPHEADER	list
CURLOPT_QUOTE		list
CURLOPT_POSTQUOTE	list

CURLOPT_HEADER		bool
CURLOPT_NOBODY		bool
CURLOPT_PUT		bool
CURLOPT_UPLOAD		bool
CURLOPT_FILE		file	w 'filename'
CURLOPT_WRITEHEADER	file	w 'filename'
CURLOPT_INFILE		file	r 'filename'
CURLOPT_STDERR		file	w 'filename'
EOS

template = Hash.new

template['setter'] = <<EOS
   rb_define_method(cCurl, "<NAME>=", set_<NAME>, 1);
EOS
template['getter'] = <<EOS
   rb_define_method(cCurl, "<NAME>", get_<NAME>, 0);
EOS

template['string'] = <<'EOS'
static VALUE
set_<NAME>(VALUE self, VALUE v)
{
    rbCurl *s;
    char   *src, *dst;
    int    len;
    Data_Get_Struct(self, rbCurl, s);
    rb_hash_aset(s->prms, rb_str_new2("<NAME>"), v);  /* keep obj */

    src = rb_str2cstr(v, &len);   /* copy string */
    dst = ALLOC_N(char, len+1);   /* able to free */
    memcpy(dst, src, len);
    dst[len] = '\0';

    curl_easy_setopt(s->curl, <ARGV>, dst);
    return Qnil;
}
static VALUE
get_<NAME>(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    return rb_hash_aref(s->prms, rb_str_new2("<NAME>"));
}

EOS

template['long'] = <<'EOS'
static VALUE
set_<NAME>(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    if (TYPE(v) != T_FIXNUM) { rb_raise(rb_eArgError, "must be Fixnum"); }
    rb_hash_aset(s->prms, rb_str_new2("<NAME>"), v);
    curl_easy_setopt(s->curl, <ARGV>, FIX2LONG(v));
    return Qnil;
}
static VALUE
get_<NAME>(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    return rb_hash_aref(s->prms, rb_str_new2("<NAME>"));
}

EOS

template['bool'] = <<'EOS'
static VALUE
set_<NAME>(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    rb_hash_aset(s->prms, rb_str_new2("<NAME>"), v);

    switch (v) {
    case Qtrue:  { curl_easy_setopt(s->curl, <ARGV>, TRUE); break; }
    case Qfalse: { curl_easy_setopt(s->curl, <ARGV>, FALSE); break; }
    default:	 { rb_raise(rb_eArgError, "must be true or false"); }
    }
    return Qnil;
}
static VALUE
get_<NAME>(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    return rb_hash_aref(s->prms, rb_str_new2("<NAME>"));
}

EOS

template['list'] = <<'EOS'
static VALUE
set_<NAME>(VALUE self, VALUE v)
{
    rbCurl *s;
    int i;
    Data_Get_Struct(self, rbCurl, s);
    rb_hash_aset(s->prms, rb_str_new2("<NAME>"), v);  /* keep obj */

    curl_slist_free_all(s->list_<NAME>);
    for (i = 0; i < RARRAY(v)->len; i++) {
        s->list_<NAME> = curl_slist_append(s->list_<NAME>, rb_str2cstr(RARRAY(v)->ptr[i], 0));
    }
    return Qnil;
}
static VALUE
get_<NAME>(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    return rb_hash_aref(s->prms, rb_str_new2("<NAME>"));
}

EOS

template['file'] = <<'EOS'
static VALUE
set_<NAME>(VALUE self, VALUE v)
{
    rbCurl *s;
    int i;
    FILE *f;
    Data_Get_Struct(self, rbCurl, s);
    rb_hash_aset(s->prms, rb_str_new2("<NAME>"), v);  /* keep obj */

    f = fopen(rb_str2cstr(v, 0), "<OPT>");
    if (!f) {
        rb_raise(rb_eRuntimeError, "can't open file");
    }
    curl_easy_setopt(s->curl, <ARGV>, f);
    s->file_<NAME> = f;
    return Qnil;
}
static VALUE
get_<NAME>(VALUE self, VALUE v)
{
    rbCurl *s;
    Data_Get_Struct(self, rbCurl, s);
    return rb_hash_aref(s->prms, rb_str_new2("<NAME>"));
}

EOS

#######################################################################

prms.each {|prm|
  argv,type = prm.split
  next unless argv
  name = argv.sub('CURLOPT_','').downcase

  puts template['setter'].gsub('<NAME>', name)
  puts template['getter'].gsub('<NAME>', name)
}
puts
puts

prms.each {|prm|
  argv,type,opt = prm.split
  next unless argv
  name = argv.sub('CURLOPT_','').downcase

  puts template[type].gsub('<NAME>', name).gsub('<ARGV>', argv).gsub('<OPT>', opt)
}
puts
puts

prms.each {|prm|
  argv,type = prm.split
  next unless argv
  name = argv.sub('CURLOPT_','').downcase

  case type
  when 'bool'
    type = 'true|false'
  when 'string'
    type = 'String'
  when 'long'
    type = 'Number'
  when 'list'
    type = '[String, String,..]'
  when 'file'
    type = '"filename"'
  end

  puts "  self.#{name}= #{type}"
  puts "  self.#{name}"
}
